//
//  BFRecordScreenController.swift
//  BFRecordScreenKit_Example
//
//  Created by 胡志强 on 2021/11/23.
//  Copyright © 2021 CocoaPods. All rights reserved.
//

import Foundation
import BFUIKit
import GPUImage
import Photos
import BFCommonKit
import BFFramework
import UIKit


struct WithDrawModel {
    var type:Int   // 0:拖动； 1:预览播放暂停 2: 录音结束
    var timestamp:Double
}


public class BFRecordScreenController: BFBaseViewController {
    
    public var nextActionHandle:(()->Void)?
    public var closeActionHandle:(()->Void)?
    
    // MARK: - 录制参数
    public var assets = [PHAsset]()
    
    var currItemModelIndex = 0
    public var itemModels = [BFRecordItemModel]()
//    var shouldPlayRecordIndex:Int = -1          // 当前应该播放的录音资源序号
    var currentPlayRecordIndex:Int = -1         // >= 0 :当前正在播放的录音资源序号； -3： 刚录音完，不需要播放录音； -1:初始化阶段
    var isRecording = false {                   // 是否正在录音
        didSet{
            withDrawBtn.isHidden = isRecording
            changeVoiceBtn.isHidden = isRecording
            
            recordBtn.setTitle(isRecording ? "松手 完成" :"按住 说话", for: .normal)
            recordBtn.backgroundColor = UIColor.hexColor(hexadecimal: "#28BE67", alpha: isRecording ? 0.6 : 1)
        }
    }
    
    var isNormalPlaying = false {               // 是否正在播放
        didSet{
            playBtn.isSelected = isNormalPlaying
        }
    }
    var currentAssetProgress : CMTime = .zero   // 当前素材播放的进度
    // 视频素材
    public var avasset:AVURLAsset?
    
//    public var recordList:[PQVoiceModel] = [PQVoiceModel]()
    
    var assetPlayer:AVPlayer?       // 原视频音频播放器
    var isCompletePlay = true
    var hadPrepareToPlayRecord = false    // 录音播放器准备
    var recordPlayer:AVPlayer? // 录音音频播放器
    var movie :GPUImageMovie?       // 视频预览
    var playView :GPUImageView?     // 视频展示视图
    
    // MARK: 行为参数
    var events = [WithDrawModel]()              // 行为记录，方便撤销
    var isDragingProgressSlder : Bool = false   // 是否在拖动进度条
    var isStopAtRecordRange = -1
    //定义音频的编码参数
    let recordSettings:[String : Any] = [AVSampleRateKey : 44100.0, //声音采样率
                            AVFormatIDKey : kAudioFormatLinearPCM,  //编码格式
                    AVNumberOfChannelsKey : 1,                      //采集音轨
                  AVEncoderBitDepthHintKey: 16,                     // 位深
                 AVEncoderAudioQualityKey : AVAudioQuality.medium.rawValue] //音频质量

    
    // 录音相关
    lazy var recorderManager : BFVoiceRecordManager = {
        
        let manager = BFVoiceRecordManager()
        manager.cancelRecordHandle = { error in
            
        }
        manager.endRecordHandle = {[weak self] (model, error) in
            if let sself = self, let model = model, FileManager.default.fileExists(atPath: model.wavFilePath ?? ""){
                // 加入到语音数组里
                // TODO: 原逻辑要删除新录音后边的数据， 新逻辑是覆盖则删除
                var index = sself.itemModels[sself.currItemModelIndex].voiceStickers.count - 1
                while index >= 0{
                    let m = sself.itemModels[sself.currItemModelIndex].voiceStickers[index]
                    index -= 1
                    if model.endTime > m.startTime && model.endTime <= m.endTime
                        || model.startTime <= m.startTime && model.startTime > m.endTime{
                        sself.itemModels[sself.currItemModelIndex].voiceStickers.remove(at: index+1)
                        continue
                    }
                    if model.startTime < m.endTime {
                        break
                    }
                }
                BFLog(1, message: "添加录音文件:\(model.startTime) -- \(model.endTime)")
                
                sself.itemModels[sself.currItemModelIndex].voiceStickers.append(model)
                
                sself.drawOrUpdateRecordProgessLable()
                
                sself.currentPlayRecordIndex = -3 // 刚录音完，不需要播放录音
            }
            
        }
        return manager
    }()

    // MARK: - 视图参数
    var beginOnStartBtn:Bool = false
    var touchStart:CGPoint = CGPoint(x: 0, y: 0)
    var avplayerTimeObserver: NSKeyValueObservation?

    lazy var progreddL : UILabel = {
        let l = UILabel()
        l.textAlignment = .center
        l.font = UIFont.systemFont(ofSize: 10)
        l.textColor = .white
        l.shadowColor = .black
        l.shadowOffset = CGSize(width: 1, height: 1)
        return l
    }()
    
    lazy var playBtn:UIButton = {
        let btn = UIButton(frame: view.bounds)
        btn.addTarget(self, action: #selector(playVideo(btn:)), for: .touchUpInside)
        return btn
    }()
    
    lazy var bottomeView:UIImageView = {
        let iv = UIImageView(image: imageInRecordScreenKit(by: "bottom_shadow"))
        iv.contentMode = .scaleAspectFill
        iv.isUserInteractionEnabled = true
        // 拖曳手勢
        let pan = UIPanGestureRecognizer(target: self, action: #selector(pan(recognizer:)))
        pan.delegate = self
        pan.minimumNumberOfTouches = 1
        pan.maximumNumberOfTouches = 1
        iv.addGestureRecognizer(pan)
        
        return iv
    }()
    
    lazy var recordBtn:UIButton = {
        let btn = UIButton(type: .custom)
        btn.backgroundColor = ThemeStyleColor
        btn.setTitle("按住 说话", for: .normal)
        btn.adjustsImageWhenHighlighted = false
        btn.addTarget(self, action: #selector(startRecord), for: .touchDown)
        btn.addTarget(self, action: #selector(endRecord), for: .touchUpInside)

        return btn
    }()
    
//    lazy var progessSildeBackV : UIView = {
//        let vv = UIView()
//        vv.backgroundColor = .orange // .clear
//
//        return vv
//    }()
    
    lazy var withDrawBtn:UIButton = {
        let btn = UIButton(type: .custom)
        btn.setImage(imageInRecordScreenKit(by: "withdraw_n"), for: .normal)
        btn.setImage(imageInRecordScreenKit(by: "withdraw_h"), for: .highlighted)
        btn.setTitle("撤销", for: .normal)
        btn.setTitleColor(.white, for: .normal)
        btn.setTitleColor(.gray, for: .highlighted)
        btn.titleLabel?.font = UIFont.systemFont(ofSize: 12)
        btn.contentVerticalAlignment = UIControl.ContentVerticalAlignment.center;
        btn.addTarget(self, action: #selector(withdrawAction), for: .touchUpInside)
        return btn
    }()
    
    lazy var changeVoiceBtn:UIButton = {
        let btn = UIButton(type: .custom)
        btn.setImage(imageInRecordScreenKit(by: "changeVoice_n"), for: .normal)
        btn.setImage(imageInRecordScreenKit(by: "changeVoice_h"), for: .highlighted)
        btn.setTitle("变声", for: .normal)
        btn.setTitleColor(.white, for: .normal)
        btn.setTitleColor(ThemeStyleColor, for: .highlighted)
        btn.titleLabel?.font = UIFont.systemFont(ofSize: 12)
        btn.contentVerticalAlignment = UIControl.ContentVerticalAlignment.center;
        btn.addTarget(self, action: #selector(changeVoiceAction), for: .touchUpInside)
        return btn
    }()
    
    lazy var toolV : BFIntroduceToolView = {
        let toolV = BFIntroduceToolView()
        toolV.centerY = view.centerY
        
        toolV.choosedToolHandle = {[weak self] tool in
            guard let sself = self else {
                return
            }
            tool.center = sself.view.center
            sself.view.addSubview(tool)
        }
        
        return toolV
        
    }()
    
    lazy var progressThumV : BFVideoThumbProgressView = {
        let vv = BFVideoThumbProgressView(frame: CGRect(x: 0, y: 54, width: cScreenWidth, height: 50))
        vv.dragScrollProgressHandle = {[weak self] isStart, process in
            DispatchQueue.main.async {[weak self] in
                guard let sself = self else {
                    return
                }
                if isStart {
                    sself.events.append(WithDrawModel(type: 0, timestamp: sself.currentAssetProgress.seconds))
                }
                if sself.isNormalPlaying || sself.isRecording {
                    sself.pause()
                }
                sself.isDragingProgressSlder = true
                sself.changeProgress(progress: process)
            }
        }
        vv.dragEndHandle = { [weak self] process in
            
            guard let sself = self else {
                return
            }
            sself.thumbViewEnded(progress: process)
        }
        vv.isHidden = true
        return vv
    }()
    
    //MARK: ------------------ 生命周期
    deinit {
        cleanMovieTarget()
        NotificationCenter.default.removeObserver(self)
        avplayerTimeObserver?.invalidate()
        if isRecording{
            recorderManager.stopRecord(isCancel: true)
        }
        assetPlayer?.pause()
        recordPlayer?.pause()
        
    }
    
    public override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        self.navigationController?.isNavigationBarHidden = true
        hiddenNavigation()
    }
    
    public  override func viewDidLoad(){
        super.viewDidLoad()
        _ = disablePopGesture()
        
        view.backgroundColor = .lightGray
        
        playView = GPUImageView(frame: view.bounds)
        view.addSubview(playView!)
        fetchVideo()
        
        view.addSubview(playBtn)
        view.addSubview(bottomeView)
        bottomeView.addSubview(progreddL)
//        view.addSubview(toolV)
        bottomeView.addSubview(recordBtn)
        bottomeView.addSubview(withDrawBtn)
        bottomeView.addSubview(changeVoiceBtn)
        bottomeView.addSubview(progressThumV)
//        progressThumV.addSubview(progessSildeBackV)

        if checkStatus() {
            try? AVAudioSession.sharedInstance().setCategory(.playAndRecord, options: .defaultToSpeaker)
        }
        
        layoutsubview()
    }
    func layoutsubview() {

        bottomeView.snp.makeConstraints { make in
            make.left.bottom.right.equalToSuperview()
            make.height.equalTo(adapterWidth(width: 220))
        }
        
        progreddL.snp.makeConstraints { make in
            make.width.equalTo(100)
            make.centerX.equalToSuperview()
            make.bottom.equalTo(progressThumV.snp.top)
            make.height.equalTo(18)
        }
        
        withDrawBtn.snp.makeConstraints { make in
            make.left.equalToSuperview()
            make.width.height.equalTo(65)
            make.top.equalTo(128)
        }
        changeVoiceBtn.snp.makeConstraints { make in
            make.right.equalToSuperview()
            make.top.width.height.equalTo(withDrawBtn)
        }
        
        recordBtn.snp.makeConstraints { make in
            make.left.equalTo(withDrawBtn.snp.right)
            make.right.equalTo(changeVoiceBtn.snp.left)
            make.height.equalTo(42)
            make.top.equalTo(withDrawBtn).offset(6)
        }
        
//        progessSildeBackV.snp.makeConstraints { make in
//            make.left.equalToSuperview()
//            make.right.equalToSuperview()
//            make.bottom.equalToSuperview()
//            make.height.equalTo(8)
//        }
        
        withDrawBtn.imageEdgeInsets = UIEdgeInsets(top: -withDrawBtn.imageView!.height, left: 0, bottom: 0, right: -withDrawBtn.titleLabel!.width);
        withDrawBtn.titleEdgeInsets = UIEdgeInsets(top: withDrawBtn.titleLabel!.height + 2, left: -withDrawBtn.imageView!.width, bottom: 0, right: 0);
        
        changeVoiceBtn.imageEdgeInsets = UIEdgeInsets(top: -changeVoiceBtn.imageView!.height-2, left: 0, bottom: 0, right: -changeVoiceBtn.titleLabel!.width);
        changeVoiceBtn.titleEdgeInsets = UIEdgeInsets(top: changeVoiceBtn.titleLabel!.height+2, left: -changeVoiceBtn.imageView!.width, bottom: 0, right: 0);
        
    }
    
//    public override func viewWillLayoutSubviews() {
//        super.viewWillLayoutSubviews()
//
//    }
    
    // MARK: - 按钮事件响应
    
    public override func backBtnClick() {
        pause()
    }
    
    // 触发拖曳手势后，执行的动作
    @objc func pan(recognizer: UIPanGestureRecognizer) {
        // 设置 UIView 新的位置
        if !checkStatus(show: false) {
            return
        }

        let point = recognizer.location(in: bottomeView)

        switch recognizer.state {
        case .began:
            touchStart = recognizer.location(in: bottomeView)
            beginOnStartBtn = recordBtn.frame.contains(touchStart)
        case .changed:
            if beginOnStartBtn == true {
                let nowPoint = recognizer.location(in: bottomeView)
                BFLog(1, message: "nowPoint x: \(nowPoint.x) y:\(nowPoint.y)")
                if recordBtn.frame.contains(nowPoint) {
                    
                } else {
                    
                }
            }

        case .ended:
            BFLog(1, message: "移动结束 \(beginOnStartBtn)")
            if beginOnStartBtn == true {
                beginOnStartBtn = false
                if recordBtn.frame.contains(point) {
                    // 结束录制
                    endRecord()
                } else {
                    cancleRecord()
                }
            }

        default:
            break
        }
    }
    
    @objc func startRecord(){
        BFLog(1, message: "start \(UIControl.Event.touchDown)")
        isRecording = true

        pause()
        
        events.append(WithDrawModel(type: 2, timestamp: self.currentAssetProgress.seconds))

        let model = PQVoiceModel()
        model.startTime = self.currentAssetProgress.seconds
        model.volume = 100
        recorderManager.voiceModel = model
        recorderManager.startRecord(index: 1)
//        movie?.startProcessing()
        assetPlayer?.volume = 0
        assetPlayer?.play()
    }
    
    @objc func endRecord(){
        
        isRecording = false
        // 存储录音
        recorderManager.voiceModel?.endTime = self.currentAssetProgress.seconds
        recorderManager.endRecord()

        pause()
    }
    
    func cancleRecord(){

        isRecording = false
        recorderManager.cancleRecord()
        
        pause()
    }
    
    @objc func withdrawAction(){
        pause()
        if let action = events.last {
            var jumpTime = action.timestamp
            if action.type == 2 {
                // 撤销录制
                if let modelIndex = itemModels[currItemModelIndex].voiceStickers.firstIndex(where: { mod in
                    mod.startTime == action.timestamp
                }) {
                    let model = itemModels[currItemModelIndex].voiceStickers[modelIndex]
                    itemModels[currItemModelIndex].voiceStickers.remove(at: modelIndex)
                    drawOrUpdateRecordProgessLable()
                    jumpTime = model.startTime
                }
            }else {
            }
            events.removeLast()
            if let dur = itemModels[currItemModelIndex].baseMaterial?.duration.seconds,dur > 0 {
                changeProgress(progress: Float(jumpTime / dur))
                isDragingProgressSlder = false
                currentPlayRecordIndex = -1
                hadPrepareToPlayRecord = false
                progressThumV.progress = jumpTime
            }
        }
    }
    
    @objc func changeVoiceAction(){
//        nextActionHandle?()
        pause()
    }
    
    @objc func playVideo(btn:UIButton){
        if btn.isSelected {
            pause()
            searchStopAtRecordRange()
        }else {
            events.append(WithDrawModel(type: 1, timestamp: self.currentAssetProgress.seconds))
            play()
        }
    }

    @objc func sliderTouchBegan(sender _: UISlider) {
        isDragingProgressSlder = true
        pause()

    }

    @objc func sliderTouchEnded(sender: UISlider) {
        changeProgress(progress: sender.value)
        isDragingProgressSlder = false
        currentPlayRecordIndex = -1
        hadPrepareToPlayRecord = false
    }
    @objc func sliderValueDidChanged(sender: UISlider) {
        changeProgress(progress: sender.value)
    }
    
    func thumbViewEnded(progress:Float) {
        changeProgress(progress: progress)
        isDragingProgressSlder = false
        currentPlayRecordIndex = -1
        hadPrepareToPlayRecord = false
        
        searchStopAtRecordRange()
    }
    
    func searchStopAtRecordRange() {
        // TODO: 判断是否停止录音区间，是则删除相关录音，画笔，头像，字幕
        let elems = itemModels[currItemModelIndex].voiceStickers.enumerated().filter({ elem in
            elem.1.startTime <= self.currentAssetProgress.seconds && elem.1.endTime > self.currentAssetProgress.seconds
        })
        if elems.count > 0{
            //  TODO: 停在了录音区间,显示删除按钮
            isStopAtRecordRange = elems.first!.0
            BFLog(1, message: "停在了录音区间 里")
        }else {
            isStopAtRecordRange = -1
            BFLog(1, message: "停在了录音区间 外")
        }
    }
    
    // MARK: - 权限申请
    func checkStatus(show: Bool = true) -> Bool {
        let status = AVCaptureDevice.authorizationStatus(for: .audio)
        switch status {
        case .denied, .restricted:

            BFLog(message: "麦克风权限被拒绝，请在设置中打开")
            if show {
                let remindData = BFBaseModel()
                remindData.title = "票圈视频需要访问你的麦克风才能录音"
                remindData.summary = ""
                let remindView = BFRemindView(frame: CGRect(x: 0, y: 0, width: cScreenWidth, height: cScreenHeigth))
                remindView.isBanned = true
                remindView.confirmBtn.setTitle("去设置", for: .normal)
                UIApplication.shared.keyWindow?.addSubview(remindView)
                remindView.remindData = remindData
                remindView.remindBlock = { item, _ in
                    if item.tag == 2 {
                        openAppSetting()
                    }
                }
            }
            return false
        case .authorized:
            return true
        case .notDetermined:
            if show {
                requestMicroPhoneAuth()
            }
            return false
        default:
            break
        }

        return false
    }
    
    func requestMicroPhoneAuth() {
        AVCaptureDevice.requestAccess(for: .audio) { granted in
            if !granted {
                BFLog(message: "麦克风权限被拒绝，请在设置中打开")
            }
        }
    }
    
    // MARK: - 音视频处理
    func playRecord(at currentT:CMTime){
        if currentPlayRecordIndex == -3 { // 刚录音完，不需要播放
            return
        }
        
        let (shouldPlayRecordIndex, recordedAudio) = itemModels[currItemModelIndex].voiceStickers.enumerated().first { model in
            model.1.endTime > CMTimeGetSeconds(currentT)
        } ?? (-1, nil)
        
        guard let recordedAudio = recordedAudio else {
            return
        }
        
        BFLog(1, message: "当前时间：\(CMTimeGetSeconds(currentT)), 找到的音频：\(recordedAudio.startTime), \(recordedAudio.endTime)")
        
        // 创建播放器
        if self.recordPlayer == nil || (self.recordPlayer?.currentItem?.asset as? AVURLAsset)?.url.lastPathComponent != (recordedAudio.wavFilePath as NSString).lastPathComponent {
            let newItem = AVPlayerItem(url: URL(fileURLWithPath: recordedAudio.wavFilePath))
            if let player = self.recordPlayer {
                player.pause()
                if let playItem = player.currentItem {
                    NotificationCenter.default.removeObserver(self, name: .AVPlayerItemDidPlayToEndTime, object: playItem)
                    recordPlayer?.replaceCurrentItem(with: newItem)
                }
            }else {
                self.recordPlayer = AVPlayer(playerItem: newItem)
            }
            self.recordPlayer!.volume = 1
            //                self.recordPlayer?.prepareToPlay()
            currentPlayRecordIndex = -1
            hadPrepareToPlayRecord = false
            BFLog(1, message: "录音播放器初始化(有时候不准)")
            
            NotificationCenter.default.addObserver(forName: .AVPlayerItemDidPlayToEndTime, object: newItem, queue: .main) { [weak self] notify in
                self?.hadPrepareToPlayRecord = false
                self?.currentPlayRecordIndex = -1
            }
        }
        
        synced(currentPlayRecordIndex) {
            if !hadPrepareToPlayRecord
                && CMTimeGetSeconds(currentT) >= recordedAudio.startTime
                && CMTimeGetSeconds(currentT) <= recordedAudio.endTime - 0.2 // 这个条件是避免录音结束后有小幅度回退导致播放最新录音
            {
                // 应当开始播放了
                // 两个逻辑：如果在播，则跳过;如果暂停拖动到中间，则seek
                if currentPlayRecordIndex == -1 && self.isNormalPlaying{
                    let second = CMTimeGetSeconds(currentT) - recordedAudio.startTime
                    DispatchQueue.main.async {[weak self] in
                        self?.recordPlayer?.seek(to: CMTime(value: CMTimeValue(second), timescale: 100), toleranceBefore: CMTime(value: 1, timescale: 1000000), toleranceAfter: CMTime(value: 1, timescale: 1000000), completionHandler: {[weak self] finished in
                            if finished && (self?.isNormalPlaying ?? false) {
                                self?.recordPlayer?.play()
                                BFLog(1, message: "录音开始播放 playing, \(second), \(CMTimeGetSeconds(self?.recordPlayer?.currentItem?.duration ?? .zero))")

                            }
                        })
                    }
                    currentPlayRecordIndex = shouldPlayRecordIndex
                    hadPrepareToPlayRecord = true
                    BFLog(1, message: "录音开始播放2, \(second), \(CMTimeGetSeconds(recordPlayer?.currentItem?.duration ?? .zero))")
                }
            }
            
        }
        BFLog(1, message: "应当播放：\(shouldPlayRecordIndex), 当前播放：\(currentPlayRecordIndex)")
//        if let recordedAudio = recordedAudio {
//
//
//            if shouldPlayRecordIndex != currentPlayRecordIndex {
//                // 设置新的播放资源
//
////                self.recordPlayer.delegate = self
//                self.recordPlayer.play()
//
//            } else {
//                // 更新播放进度
//                let second = CMTimeGetSeconds(duration) - recordedAudio.startTime
//                self.recordPlayer.seek(to: CMTime(seconds: second, preferredTimescale: 25), toleranceBefore: CMTime(value: 1, timescale: 1000), toleranceAfter: CMTime(value: 1, timescale: 1000))
//            }
//        }
    }
    
    func play(){
        BFLog(1, message: "开始播放 \(self.currentAssetProgress.seconds)")
        isNormalPlaying = true
        assetPlayer?.volume = 0.2
        movie?.startProcessing()
        
        self.assetPlayer?.play()
    }
    
    func pause(){
        BFLog(1, message: "暂停播放")
        isNormalPlaying = false
//        movie?.cancelProcessing()
        assetPlayer?.pause()
        recordPlayer?.pause()
        
        assetPlayer?.seek(to: self.currentAssetProgress , toleranceBefore: CMTime(value: 1, timescale: 1000000), toleranceAfter: CMTime(value: 1, timescale: 1000000), completionHandler: { finished in
        })
    }
    
    func fetchVideo(){
        if self.assets.count > 0 {

            currItemModelIndex = 0
            
            for (index, asset) in self.assets.enumerated() {
                let itemModel = BFRecordItemModel()
                itemModel.index = 0
                itemModel.width = asset.pixelWidth
                itemModel.height = asset.pixelHeight
                
                itemModels.append(itemModel)
                
                let options = PHVideoRequestOptions()
                options.isNetworkAccessAllowed = true
                options.deliveryMode = .automatic
                
                PHImageManager.default().requestPlayerItem(forVideo:asset, options: options, resultHandler: { [weak self] playerItem, info in

                    guard let item = playerItem else {
                        cShowHUB(superView: nil, msg: "视频获取失败")
                        return
                    }
                    if index == 0 {
                        self?.setAudioPlay(item: item)
                        self?.setVideoPlay(item: item)
                    }
                })
                
//                let option = PHImageRequestOptions()
//                option.isNetworkAccessAllowed = true //允许下载iCloud的图片
//                option.resizeMode = .fast
//                option.deliveryMode = .highQualityFormat
//                PHImageManager.default().requestImage(for: asset,
//                                               targetSize: self.view.bounds.size,
//                                              contentMode: .aspectFit,
//                                                  options: option)
//                { (image, nil) in
//                     // 设置首帧/封面
//                    if image != nil {
//                        let pic = GPUImagePicture(image: image)
//                        let filet = GPUImageFilter()
//                        pic?.addTarget(filet)
//                        filet.addTarget(self.playView)
//                        pic?.processImage()
//                    }
//                }
                
                PHCachingImageManager().requestAVAsset(forVideo: asset, options: options, resultHandler: {[weak self] (asset: AVAsset?, audioMix: AVAudioMix?, info) in
                    if let urlasset = asset as? AVURLAsset {
                        self?.avasset = urlasset
                        itemModel.baseMaterial = urlasset
                        DispatchQueue.main.async {[weak self] in
                            self?.progressThumV.videoAsset = urlasset
                            self?.progressThumV.isHidden = false
                        }
                    }
                })
            }
        }

    }
    
    func setVideoPlay(item:AVPlayerItem){
        if movie != nil {
            cleanMovieTarget()
        }
        movie = GPUImageMovie(playerItem: item)
//        movie?.runBenchmark = true
        movie?.playAtActualSpeed = true

        let filter = GPUImageFilter()
        movie?.addTarget(filter)
        filter.addTarget(playView)
        
        movie?.startProcessing()
    }
    
    func setAudioPlay(item:AVPlayerItem){

        if let playItem = assetPlayer?.currentItem {
            NotificationCenter.default.removeObserver(self, name: .AVPlayerItemDidPlayToEndTime, object: playItem)
            assetPlayer?.replaceCurrentItem(with: item)
        }else {
            assetPlayer = AVPlayer(playerItem: item)
            avplayerTimeObserver = assetPlayer?.addPeriodicTimeObserver(forInterval: CMTime(value: 1, timescale: 100), queue: DispatchQueue.global()) {[weak self] time in
             //    进度监控
                
                self?.currentAssetProgress = time
                BFLog(1, message: "curr:\(CMTimeGetSeconds(time))")
                if CMTimeGetSeconds(item.duration) > 0 {
                    DispatchQueue.main.async { [weak self] in
                        self?.progreddL.text = String(format: "%.2f", CMTimeGetSeconds(time), CMTimeGetSeconds(item.duration))
                        if !(self?.isDragingProgressSlder ?? false){
                            self?.progressThumV.progress = time.seconds
                        }
                    }
                }
                
                if (self?.isNormalPlaying ?? false) || (self?.isRecording ?? false) {
                    // 播放对应的录音音频
                    self?.playRecord(at: time)
                }
                

            } as? NSKeyValueObservation
        }

        
        NotificationCenter.default.addObserver(forName: .AVPlayerItemDidPlayToEndTime, object: assetPlayer?.currentItem, queue: .main) { [weak self] notify in
            BFLog(1, message: "AVPlayerItemDidPlayToEndTime = \(notify)")
            self?.isNormalPlaying = false
            self?.assetPlayer?.seek(to: CMTime.zero)
            self?.progressThumV.progress = 0
            
            self?.currentPlayRecordIndex = -1
            if self?.isRecording ?? false {
                self?.endRecord()
            }
        }
    }

    
    func cleanMovieTarget(){
        movie?.cancelProcessing()
        movie?.targets().forEach({ target in
            if let objc = target as? GPUImageOutput{
                objc.removeAllTargets()
            }
        })
        movie?.removeAllTargets()
        movie?.removeFramebuffer()
        GPUImageContext.sharedFramebufferCache().purgeAllUnassignedFramebuffers()
        
    }
    
    //MARK: - 录音对应图像绘制
    
    func changeProgress(progress:Float) {
        if let duration = assetPlayer?.currentItem?.duration {
            self.currentAssetProgress = CMTime(value: CMTimeValue(progress * Float(CMTimeGetSeconds(duration)) * 100), timescale: 100)
            DispatchQueue.main.async {[weak self] in
                self!.progreddL.text = String(format: "%.2f", CMTimeGetSeconds(self!.currentAssetProgress))
            }

            assetPlayer!.seek(to: self.currentAssetProgress, toleranceBefore: CMTime(value: 1, timescale: 1000000), toleranceAfter: CMTime(value: 1, timescale: 1000000)) { finished in

            }
        }
    }
    
    func drawOrUpdateRecordProgessLable(){
        DispatchQueue.main.async {[weak self] in
            guard let sself = self else {
                return
            }
            sself.progressThumV.progessIndicateBackV.subviews.forEach { vv in
                vv.removeFromSuperview()
            }
            
            if let totalDur = sself.itemModels[sself.currItemModelIndex].baseMaterial?.duration.seconds, totalDur > 0, sself.itemModels[sself.currItemModelIndex].voiceStickers.count > 0 {
                let width = sself.progressThumV.progessIndicateBackV.width
                let height = sself.progressThumV.progessIndicateBackV.height
                sself.itemModels[sself.currItemModelIndex].voiceStickers.forEach { model in
                    let lineV = UIView(frame: CGRect(x: model.startTime * width / totalDur , y: 0, width: (model.endTime - model.startTime) * width / totalDur, height: height))
                    lineV.backgroundColor = ThemeStyleColor
                    sself.progressThumV.progessIndicateBackV.addSubview(lineV)
                }
            }
        }
    }
}

extension BFRecordScreenController:GPUImageMovieDelegate {
    public func didCompletePlayingMovie(){
        BFLog(1, message: "播放结束")
        currentPlayRecordIndex = -1
    }
}

extension BFRecordScreenController:AVAudioRecorderDelegate {
    public func audioRecorderDidFinishRecording(_ recorder: AVAudioRecorder, successfully flag: Bool) {
        BFLog(1, message: "录音结束")
    }
}

extension BFRecordScreenController : AVAudioPlayerDelegate {
    public func audioPlayerDidFinishPlaying(_ player: AVAudioPlayer, successfully flag: Bool) {
        BFLog(1, message: "录音播放结束")
    }
}
